==> 0.0.2.tar.gz: lib/services/stateMachine/stateMachine.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:origami_telegram/models/serviceRequest.dart';
import 'package:origami_telegram/models/telegramResponseModel.dart';
import 'package:origami_telegram/services/stateMachine/serviceCall.dart';
import 'package:origami_telegram/services/stateMachine/state.dart';
import 'package:origami_telegram/services/stateMachine/statePath.dart';
import 'package:origamicore/models/messageModel.dart';
import 'package:origamicore/routing/router.dart';
 
class StateMachine
{
  late List<StatePath> pathes;
  late List<State> states;

  Map<String,State> stateById={};
  Map<String,State> stateByName={};
  StateMachine();
  Future fromFile(String path) async
  {
    var data=json.decode(await File(path).readAsString());
    pathes=StatePath.fromJsonList(data['pathes']);
    states=State.fromJsonList(data['states']);
    for(var a in states) 
    {
      stateById[a.id]=a;
      stateByName[a.name]=a;
    }

  }
  StateMachine.fromJson(dynamic data)
  {
    pathes=StatePath.fromJsonList(data['pathes']);
    states=State.fromJsonList(data['states']);
    for(var a in states) 
    {
      stateById[a.id]=a;
      stateByName[a.name]=a;
    }
  }
  dynamic matchRouteToParams(String text,String router,dynamic staticObject)
  {
    dynamic obj={};
    if(staticObject!=null)
    {
      obj=json.decode(json.encode(staticObject));
    }
    if(text=='' || router=='')return obj;
    var splitData=text.split('_');
    var names= router.split('/');
    for(var a=0;a<names.length;a++)
    {
      if(splitData.length>a)
      {
        obj[names[a]]=splitData[a];
      }
    }
    return obj;
  }
  ServiceRequest textToObject(String text)
  {
    var req= ServiceRequest();
    var split=text.split(' ');
    var name=split[0];
    var state = stateByName[name];
    if(state==null)
    {
      return req;
    }
    req.state=state ;
    var obj={'type':name.replaceAll('/', '')};
    if(split.length>1)
    {
      var splitData=split[1].split('_');
      var names= req.state.router.split('/');
      for(var a=0;a<names.length;a++)
      {
        if(splitData.length>a)
        {
          obj[names[a]]=splitData[a];
        }
      }
    }
    req.data=obj;
    req.isValid=true;
    return req;
  }
  Future<TelegramResponseModel> runState(ServiceRequest req,dynamic session)async
  { 
    
    if(req.state.service!=null)
    {
      var srv=req.state.service??ServiceCall();
      var message = await Router.run(srv.domain, srv.service, MessageModel(internalData:req.data,session:session)); 
      if(message.response!=null)
      {
        return TelegramResponseModel.fromJson(message.response?.data);        
      }
    }
    else
    {
      var state=req.state;
      var resp=TelegramResponseModel();
      resp.button=state.button??[];
      resp.text=state.text??'';
      resp.inlineKeyboard=state.isInlineButton;
      resp.state=state.id;
      return resp;
    }
    return TelegramResponseModel();
  }
  State? findState(List<StatePath> pathes,String text)
  {
      State? nextstate;
      for(var a in pathes)
      {
        if(a.type==PathType.Epsilon)
        { 
          nextstate=stateById[a.toState];
          break;
        }
        if(a.type==PathType.Sentence)
        {
          if(a.text==text)
          {
            nextstate=stateById[a.toState];
            break;
          }
        }
        if(a.type==PathType.Expersion)
        {
          var exp = RegExp(a.text);
          var matches = exp.allMatches(text).toList();
          if(matches.isNotEmpty)
          {
            nextstate=stateById[a.toState];
            break;
          }
        }
      }
    return nextstate;
  }


  Future<ServiceRequest> findServiceRequest(String state,String text,dynamic session) async
  {
    var req=ServiceRequest();
    var stateLess= pathes.where((element) => element.fromState=='-1').toList();
    var stateLessState = findState(stateLess,text);
    var newstate='';
    if(stateLessState!=null)
    {
      req.isValid=true;
      req.state=stateLessState; 
      newstate=stateLessState.id;
      req.data=  matchRouteToParams(text,stateLessState.router,stateLessState.service?.staticData);
      req.data['text']=text;

    }
    else
    {
      if(state=='')
      {
        var state= stateByName['stateNotFound']??State();
        req.state=state;
        newstate=state.id;
        req.data={};
      }
      else{
        var findPath = pathes.where((element) => element.fromState==state).toList();
        var nextstate=findState(findPath,text);
        if(nextstate!=null)
        {
          newstate=nextstate.id;
          req.isValid=true;
          req.state=nextstate; 
          req.data=  matchRouteToParams(text,nextstate.router,nextstate.service?.staticData);
          req.data['text']=text;
        }
      }

    }
    if(!req.isValid)
    {
      var state= stateByName['stateNotFound']??State();
      req.state=state;
      newstate=state.id;
      req.data={};
    }
    return req;
  }
  Future<TelegramResponseModel> findTextRoute(String state,String text,dynamic session) async
  {
    var req=ServiceRequest();
    var stateLess= pathes.where((element) => element.fromState=='-1').toList();
    var stateLessState = findState(stateLess,text);
    var newstate='';
    if(stateLessState!=null)
    {
      req.isValid=true;
      req.state=stateLessState; 
      newstate=stateLessState.id;
      req.data=  matchRouteToParams(text,stateLessState.router,stateLessState.service?.staticData);
      req.data['text']=text;

    }
    else
    {
      if(state=='')
      {
        var state= stateByName['stateNotFound']??State();
        req.state=state;
        newstate=state.id;
        req.data={};
      }
      else{
        var findPath = pathes.where((element) => element.fromState==state).toList();
        var nextstate=findState(findPath,text);
        if(nextstate!=null)
        {
          newstate=nextstate.id;
          req.isValid=true;
          req.state=nextstate; 
          req.data=  matchRouteToParams(text,nextstate.router,nextstate.service?.staticData);
          req.data['text']=text;
        }
      }

    }
    if(!req.isValid)
    {
      var state= stateByName['stateNotFound']??State();
      req.state=state;
      newstate=state.id;
      req.data={};
    }
    var responseData = await runState(req,session);
    var btn=req.state.button;
    if(btn!=null)
    {
      if(responseData.button!=btn)
      {
        var a=0;
        for(var a=0;a< btn.length;a++)
        {
          responseData.button.add(btn[a]);
        }
      }
       
    }
    if(responseData.state=='')
    {
      responseData.state=newstate;
    }
    return responseData;
  }
  Future<TelegramResponseModel> runCommand(String command,dynamic session) async
  { 
    var obj=textToObject(command);

    if(!obj.isValid) {
      return TelegramResponseModel();
    } 
    var resp= await runState(obj,session);

    // resp.text='test';
    // resp.inlineKeyboard=true;
    // resp.button=[[ActionButton(action: 'test',title: 'gggg')]];
    return resp;
  }
}

==> 0.0.4.tar.gz: lib/services/stateMachine/stateMachine.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:origami_telegram/models/serviceRequest.dart';
import 'package:origami_telegram/models/telegramResponseModel.dart';
import 'package:origami_telegram/services/stateMachine/serviceCall.dart';
import 'package:origami_telegram/services/stateMachine/state.dart';
import 'package:origami_telegram/services/stateMachine/statePath.dart';
import 'package:origamicore/models/messageModel.dart';
import 'package:origamicore/routing/router.dart';
 
class StateMachine
{
  List<StatePath> pathes;
  List<State> states;

  Map<String,State> stateById={};
  Map<String,State> stateByName={};
  StateMachine();
  Future fromFile(String path) async
  {
    var data=json.decode(await File(path).readAsString());
    pathes=StatePath.fromJsonList(data['pathes']);
    states=State.fromJsonList(data['states']);
    for(var a in states) 
    {
      stateById[a.id]=a;
      stateByName[a.name]=a;
    }

  }
  StateMachine.fromJson(dynamic data)
  {
    pathes=StatePath.fromJsonList(data['pathes']);
    states=State.fromJsonList(data['states']);
    for(var a in states) 
    {
      stateById[a.id]=a;
      stateByName[a.name]=a;
    }
  }
  dynamic matchRouteToParams(String text,String router,dynamic staticObject)
  {
    dynamic obj={};
    if(staticObject!=null)
    {
      obj=json.decode(json.encode(staticObject));
    }
    if(text=='' || router=='')return obj;
    var splitData=text.split('_');
    var names= router.split('/');
    for(var a=0;a<names.length;a++)
    {
      if(splitData.length>a)
      {
        obj[names[a]]=splitData[a];
      }
    }
    return obj;
  }
  ServiceRequest textToObject(String text)
  {
    var req= ServiceRequest();
    var split=text.split(' ');
    var name=split[0];
    var state = stateByName[name];
    if(state==null)
    {
      return req;
    }
    req.state=state ;
    var obj={'type':name.replaceAll('/', '')};
    if(split.length>1)
    {
      var splitData=split[1].split('_');
      var names= req.state.router.split('/');
      for(var a=0;a<names.length;a++)
      {
        if(splitData.length>a)
        {
          obj[names[a]]=splitData[a];
        }
      }
    }
    req.data=obj;
    req.isValid=true;
    return req;
  }
  Future<TelegramResponseModel> runState(ServiceRequest req,dynamic session)async
  { 
    
    if(req.state.service!=null)
    {
      var srv=req.state.service??ServiceCall();
      var message = await Router.run(srv.domain, srv.service, MessageModel(internalData:req.data,session:session)); 
      if(message.response!=null)
      {
        return TelegramResponseModel.fromJson(message.response?.data);        
      }
    }
    else
    {
      var state=req.state;
      var resp=TelegramResponseModel();
      resp.button=state.button??[];
      resp.text=state.text??'';
      resp.inlineKeyboard=state.isInlineButton;
      resp.state=state.id;
      return resp;
    }
    return TelegramResponseModel();
  }
  State findState(List<StatePath> pathes,String text)
  {
      State nextstate;
      for(var a in pathes)
      {
        if(a.type==PathType.Epsilon)
        { 
          nextstate=stateById[a.toState];
          break;
        }
        if(a.type==PathType.Sentence)
        {
          if(a.text==text)
          {
            nextstate=stateById[a.toState];
            break;
          }
        }
        if(a.type==PathType.Expersion)
        {
          var exp = RegExp(a.text);
          var matches = exp.allMatches(text).toList();
          if(matches.isNotEmpty)
          {
            nextstate=stateById[a.toState];
            break;
          }
        }
      }
    return nextstate;
  }


  Future<ServiceRequest> findServiceRequest(String state,String text,dynamic session) async
  {
    var req=ServiceRequest();
    var stateLess= pathes.where((element) => element.fromState=='-1').toList();
    var stateLessState = findState(stateLess,text);
    var newstate='';
    if(stateLessState!=null)
    {
      req.isValid=true;
      req.state=stateLessState; 
      newstate=stateLessState.id;
      req.data=  matchRouteToParams(text,stateLessState.router,stateLessState.service?.staticData);
      req.data['text']=text;

    }
    else
    {
      if(state=='')
      {
        var state= stateByName['stateNotFound']??State();
        req.state=state;
        newstate=state.id;
        req.data={};
      }
      else{
        var findPath = pathes.where((element) => element.fromState==state).toList();
        var nextstate=findState(findPath,text);
        if(nextstate!=null)
        {
          newstate=nextstate.id;
          req.isValid=true;
          req.state=nextstate; 
          req.data=  matchRouteToParams(text,nextstate.router,nextstate.service?.staticData);
          req.data['text']=text;
        }
      }

    }
    if(!req.isValid)
    {
      var state= stateByName['stateNotFound']??State();
      req.state=state;
      newstate=state.id;
      req.data={};
    }
    return req;
  }
  Future<TelegramResponseModel> findTextRoute(String state,String text,dynamic session) async
  {
    var req=ServiceRequest();
    var stateLess= pathes.where((element) => element.fromState=='-1').toList();
    var stateLessState = findState(stateLess,text);
    var newstate='';
    if(stateLessState!=null)
    {
      req.isValid=true;
      req.state=stateLessState; 
      newstate=stateLessState.id;
      req.data=  matchRouteToParams(text,stateLessState.router,stateLessState.service?.staticData);
      req.data['text']=text;

    }
    else
    {
      if(state=='')
      {
        var state= stateByName['stateNotFound']??State();
        req.state=state;
        newstate=state.id;
        req.data={};
      }
      else{
        var findPath = pathes.where((element) => element.fromState==state).toList();
        var nextstate=findState(findPath,text);
        if(nextstate!=null)
        {
          newstate=nextstate.id;
          req.isValid=true;
          req.state=nextstate; 
          req.data=  matchRouteToParams(text,nextstate.router,nextstate.service?.staticData);
          req.data['text']=text;
        }
      }

    }
    if(!req.isValid)
    {
      var state= stateByName['stateNotFound']??State();
      req.state=state;
      newstate=state.id;
      req.data={};
    }
    var responseData = await runState(req,session);
    var btn=req.state.button;
    if(btn!=null)
    {
      if(responseData.button!=btn)
      {
        var a=0;
        for(var a=0;a< btn.length;a++)
        {
          responseData.button.add(btn[a]);
        }
      }
       
    }
    if(responseData.state=='')
    {
      responseData.state=newstate;
    }
    return responseData;
  }
  Future<TelegramResponseModel> runCommand(String command,dynamic session) async
  { 
    var obj=textToObject(command);

    if(!obj.isValid) {
      return TelegramResponseModel();
    } 
    var resp= await runState(obj,session);

    // resp.text='test';
    // resp.inlineKeyboard=true;
    // resp.button=[[ActionButton(action: 'test',title: 'gggg')]];
    return resp;
  }
}
